use actix_web::{get, web, Responder, http::header::ContentType, HttpResponse, post};
use chrono::{DateTime, ParseError};
use uuid::Uuid;

use gamenight_database::{gamenight::Gamenight, DbPool, GetConnection};

use crate::{models::{gamenight, add_gamenight_request_body::AddGamenightRequestBody, get_gamenight_request::GetGamenightRequest}, request::authorization::AuthUser};
use crate::request::error::ApiError;


impl AddGamenightRequestBody {
    pub fn into_with_user(&self, user: AuthUser) -> Result<Gamenight, ParseError> {
        Ok(Gamenight {
            datetime: DateTime::parse_from_rfc3339(&self.clone().datetime.unwrap())?.with_timezone(&chrono::Utc),
            id: Uuid::new_v4(),
            name: self.clone().name.unwrap().clone(),
            owner_id: user.id
        })
    }
}

impl From<GetGamenightRequest> for Uuid {
    fn from(value: GetGamenightRequest) -> Self {
        Uuid::parse_str(value.id.unwrap().as_str()).unwrap()
    }
}

#[get("/gamenights")]
pub async fn gamenights(pool: web::Data<DbPool>, _user: AuthUser) -> Result<impl Responder, ApiError> {
    let mut conn = pool.get_conn();
    let gamenights: Vec<gamenight_database::gamenight::Gamenight> = gamenight_database::gamenights(&mut conn)?;

    Ok(HttpResponse::Ok()
        .content_type(ContentType::json())
        .body(serde_json::to_string(&gamenights)?)
    )
}

#[post("/gamenight")]
pub async fn gamenight_post(pool: web::Data<DbPool>, user: AuthUser, gamenight_data: web::Json<AddGamenightRequestBody>) -> Result<impl Responder, ApiError> {
    let mut conn = pool.get_conn();
    
    gamenight_database::gamenight::add_gamenight(&mut conn, gamenight_data.into_with_user(user)?)?;
    
    Ok(HttpResponse::Ok())
}

#[get("/gamenight")]
pub async fn gamenight_get(pool: web::Data<DbPool>, _user: AuthUser, gamenight_data: web::Json<GetGamenightRequest>) -> Result<impl Responder, ApiError> {
    let mut conn = pool.get_conn();

    let gamenight = gamenight_database::gamenight::get_gamenight(&mut conn, gamenight_data.into_inner().into())?;
    let participants = gamenight_database::gamenight_participants::get_participants(&mut conn, &gamenight.id)?;

    let model = gamenight::Gamenight{
        id: gamenight.id.to_string(),
        datetime: gamenight.datetime.to_rfc3339(),
        name: gamenight.name,
        owner_id: gamenight.owner_id.to_string(),
        participants: Some(participants.iter().map(|x| {x.to_string()}).collect())
    };

    Ok(HttpResponse::Ok()
        .content_type(ContentType::json())
        .body(serde_json::to_string(&model)?))
}